/*
FRODO: a FRamework for Open/Distributed Optimization
Copyright (C) 2008, 2009, 2010, 2011  Thomas Leaute, Brammert Ottens & Radoslaw Szymanek

FRODO is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

FRODO is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.


How to contact the authors: 
<http://lia.epfl.ch/>

EPFL / IC / IIF / LIA
Batiment IN 
Station 14 
CH - 1015 Lausanne 
Switzerland
*/

package ch.epfl.lia.frodo.algorithms.afb.test;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import junit.extensions.RepeatedTest;
import junit.framework.TestSuite;

import org.jdom.Element;

import ch.epfl.lia.frodo.algorithms.Problem;
import ch.epfl.lia.frodo.algorithms.Solution;
import ch.epfl.lia.frodo.algorithms.XCSPparser;
import ch.epfl.lia.frodo.algorithms.StatsReporterWithConvergence.CurrentAssignment;
import ch.epfl.lia.frodo.algorithms.dpop.DPOPsolver;
import ch.epfl.lia.frodo.algorithms.dpop.test.DPOPagentTest;
import ch.epfl.lia.frodo.algorithms.afb.AFB;
import ch.epfl.lia.frodo.algorithms.reformulation.ProblemRescaler;
import ch.epfl.lia.frodo.algorithms.test.AllTests;
import ch.epfl.lia.frodo.algorithms.varOrdering.election.VariableElection;
import ch.epfl.lia.frodo.algorithms.varOrdering.linear.LinearOrdering;
import ch.epfl.lia.frodo.solutionSpaces.Addable;
import ch.epfl.lia.frodo.solutionSpaces.AddableInteger;
import ch.epfl.lia.frodo.solutionSpaces.AddableReal;

/** JUnit tests for AFB
 * @param <V> the type used for variable values
 * @param <U> the type used for utility values
 * @author Alexandra Olteanu, Thomas Leaute
 */
public class AFBagentTest <V extends Addable<V>, U extends Addable<U> > extends DPOPagentTest<V, U> {
	
	/** @return the test suite */
	public static TestSuite suite () {
		TestSuite suite = new TestSuite ("Random tests for AFBagent");
		
		TestSuite tmp = new TestSuite ("Tests using QueueIOPipes");
			
		tmp.addTest(new RepeatedTest (new AFBagentTest<AddableInteger, AddableInteger> ("AFBagent.xml", true, false, false, false, AddableInteger.class, AddableInteger.class, false, +1), 5000));
		suite.addTest(tmp);
		
		tmp = new TestSuite ("Tests using QueueIOPipes without XCSP");
		tmp.addTest(new RepeatedTest (new AFBagentTest<AddableInteger, AddableInteger> ("AFBagent.xml", false, false, false, false, AddableInteger.class, AddableInteger.class, false, +1), 5000));
		suite.addTest(tmp);
		
		tmp = new TestSuite ("Tests using QueueIOPipes on minimization problems with unrestricted cost signs");
		tmp.addTest(new RepeatedTest (new AFBagentTest<AddableInteger, AddableInteger> ("AFBagent.xml", true, false, false, false, AddableInteger.class, AddableInteger.class, false, 0), 5000));
		suite.addTest(tmp);
		
		tmp = new TestSuite ("Tests using QueueIOPipes on minimization problems with unrestricted cost signs without XCSP");
		tmp.addTest(new RepeatedTest (new AFBagentTest<AddableInteger, AddableInteger> ("AFBagent.xml", false, false, false, false, AddableInteger.class, AddableInteger.class, false, 0), 5000));
		suite.addTest(tmp);
		
		tmp = new TestSuite ("Tests using QueueIOPipes on maximization problems with unrestricted utility signs");
		tmp.addTest(new RepeatedTest (new AFBagentTest<AddableInteger, AddableInteger> ("AFBagent.xml", true, false, false, false, AddableInteger.class, AddableInteger.class, true, 0), 5000));
		suite.addTest(tmp);
		
		tmp = new TestSuite ("Tests using QueueIOPipes on maximization problems with unrestricted utility signs without XCSP");
		tmp.addTest(new RepeatedTest (new AFBagentTest<AddableInteger, AddableInteger> ("AFBagent.xml", false, false, false, false, AddableInteger.class, AddableInteger.class, true, 0), 5000));
		suite.addTest(tmp);
		
		tmp = new TestSuite ("Tests using QueueIOPipes and real-valued variables");
		tmp.addTest(new RepeatedTest (new AFBagentTest<AddableReal, AddableInteger> ("AFBagent.xml", true, false, false, false, AddableReal.class, AddableInteger.class, false, +1), 5000));
		suite.addTest(tmp);
		
		tmp = new TestSuite ("Tests using QueueIOPipes and real-valued utilities");
		tmp.addTest(new RepeatedTest (new AFBagentTest<AddableInteger, AddableReal> ("AFBagent.xml", true, false, false, false, AddableInteger.class, AddableReal.class, false, +1), 5000));
		suite.addTest(tmp);
		
		tmp = new TestSuite ("Tests using QueueIOPipes and real-valued utilities without XCSP");
		tmp.addTest(new RepeatedTest (new AFBagentTest<AddableInteger, AddableReal> ("AFBagent.xml", false, false, false, false, AddableInteger.class, AddableReal.class, false, +1), 5000));
		suite.addTest(tmp);
		
		tmp = new TestSuite ("Tests using QueueIOPipes with the central mailer");
		tmp.addTest(new RepeatedTest (new AFBagentTest<AddableInteger, AddableInteger> ("AFBagent.xml", true, false, true, false, AddableInteger.class, AddableInteger.class, false, +1), 10000));
		suite.addTest(tmp);
		
		tmp = new TestSuite ("Tests using QueueIOPipes counting NCCCs");
		tmp.addTest(new RepeatedTest (new AFBagentTest<AddableInteger, AddableInteger> ("AFBagent.xml", true, false, false, true, AddableInteger.class, AddableInteger.class, false, +1), 1000));
		suite.addTest(tmp);
		
		tmp = new TestSuite ("Tests using QueueIOPipes counting NCCCs without XCSP");
		tmp.addTest(new RepeatedTest (new AFBagentTest<AddableInteger, AddableInteger> ("AFBagent.xml", false, false, false, true, AddableInteger.class, AddableInteger.class, false, +1), 1000));
		suite.addTest(tmp);
	
		tmp = new TestSuite ("Tests using TCP pipes");
		tmp.addTest(new RepeatedTest (new AFBagentTest<AddableInteger, AddableInteger> ("AFBagent.xml", true, true, false, false, AddableInteger.class, AddableInteger.class, false, +1), 1000));
		suite.addTest(tmp);
		
		return suite;
	}

	/** The stats gatherer */
	private AFB<V, U> AFBmodule;
	
	/** The name of the agent configuration file */
	private String agentFile;
	
	/** The restriction on the sign of the costs/utilities */
	private final int sign;
	
	/** Constructor
	 * @param agentFile 		the name of the agent configuration file
	 * @param useXCSP 			whether to use XCSP
	 * @param useTCP 			whether TCP pipes should be used for communication between agents
	 * @param useCentralMailer	\c true when the central mailer should be tested
	 * @param countNCCCs 		whether to count NCCCs
	 * @param domClass 			class used for variable values
	 * @param utilClass 		class used for utility values
	 * @param maximize 			Whether we should maximize or minimize
	 * @param sign 				The restriction on the sign of the costs/utilities
	 */
	@SuppressWarnings("unchecked")
	public AFBagentTest (String agentFile, boolean useXCSP, boolean useTCP, boolean useCentralMailer, boolean countNCCCs, Class<V> domClass, Class<U> utilClass, boolean maximize, int sign) {
		super (useXCSP, useTCP, useCentralMailer, false, domClass, utilClass, null, 
				(Class<? extends XCSPparser<V, U>>) XCSPparser.class, false, false, countNCCCs, false, false);
		
		super.maximize = maximize;
		this.sign = sign;
		this.agentFile = agentFile;
	}
	
	/** @see DPOPagentTest#setUp() */
	@SuppressWarnings("unchecked")
	@Override
	public void setUp () throws Exception {
		super.setUp();
		
		problemDoc = AllTests.generateProblem(graph, this.maximize, this.sign); 
		problem = new XCSPparser<V, U> (problemDoc, this.countNCCCs, false, true);
		problem.setDomClass(super.domClass);
		problem.setUtilClass(utilClass);
		if (! super.useXCSP) {
			Problem<V, U> prob = new Problem<V, U> (this.maximize, true);
			prob.reset(this.problem);
			this.problem = prob;
			problem.setDomClass(super.domClass);
			problem.setUtilClass(utilClass);
		}
		
		super.agentConfig = XCSPparser.parse("src/ch/epfl/lia/frodo/algorithms/afb/" + agentFile, false);
		
		// Set the number of rounds of VariableElection, and whether we should count NCCCs
		for (Element module : (List<Element>) agentConfig.getRootElement().getChild("modules").getChildren()) {
			String className = module.getAttributeValue("className");
			
			if (className.equals(VariableElection.class.getName()))
				module.setAttribute("nbrSteps", Integer.toString(problem.getNbrVars()));
			
			else if (className.equals(AFB.class.getName())) {
				module.setAttribute("convergence", "true");
			}
			
			else if ((this.maximize || this.sign <= 0) && className.equals(ProblemRescaler.class.getName())) 
				module.setAttribute("shift", "1000000");
		}
		
		// Create the stats gatherer
		this.AFBmodule = new AFB<V, U> (null, problem);
		this.AFBmodule.setSilent(true);
		this.AFBmodule.getStatsFromQueue(super.queue);
		
		// Listen for the linear order on variables
		LinearOrdering module = new LinearOrdering (null, problem);
		module.setSilent(true);
		module.getStatsFromQueue(queue);
	}
	
	/** @see DPOPagentTest#tearDown() */
	@Override
	protected void tearDown () throws Exception {
		super.tearDown();
		this.AFBmodule = null;
	}

	/** @see DPOPagentTest#checkOutput() */
	@Override
	protected void checkOutput() {
		Solution<V, U> solDPOP = new DPOPsolver<V, U> (super.domClass, super.utilClass).solve(super.problem);
		
		// Check the utility
		U utilDPOP = solDPOP.getUtility();
		U utilAFB = this.AFBmodule.getOptCost();
		assertEquals (utilDPOP, utilAFB);
		
		// If the problem is feasible, check the assignments
		if (utilAFB != this.problem.getPlusInfUtility() && utilAFB != this.problem.getMinInfUtility()) 
		{
			assertEquals (utilAFB, this.problem.getUtility(this.AFBmodule.getOptAssignments()).getUtility(0));

			// Check that each variable has an assignment history
			HashMap< String, ArrayList< CurrentAssignment<V> > > histories = this.AFBmodule.getAssignmentHistories();
			for (String var : super.problem.getVariables()) 
				assertFalse ("Variable " + var + " has no assignment history:\n" + histories, histories.get(var).isEmpty());
		}
	}	
}